import { ElMessage } from 'element-plus';
const ruleReg = {
    // 手机号校验
    phoneReg(phone : any) {
        const phonereg = /^[1][3,4,5,7,8][0-9]{9}$/;
        let isValid = phonereg.test(phone);
        return isValid;
    },
    // 必须包含四种的组合密码
    passwordReg4(password : any) {
        let passwordreg = /(?=.*\d)(?=.*[a-zA-Z])(?=.*[^a-zA-Z0-9]).{6,12}/;
        let isValid = passwordreg.test(password);
        return isValid;
    },
    // 必须包含四种中三种的组合校验
    passwordReg3(password : any) {
        let passwordreg = /^(?![a-zA-Z]+$)(?![A-Z0-9]+$)(?![A-Z\W_!@#$%^&*`~()-+=]+$)(?![a-z0-9]+$)(?![a-z\W_!@#$%^&*`~()-+=]+$)(?![0-9\W_!@#$%^&*`~()-+=]+$)[a-zA-Z0-9\W_!@#$%^&*`~()-+=]{6,12}$/;
        let isValid = passwordreg.test(password);
        return isValid;
    },
    // 至少六个字符，至少一个字母和一个数字
    passwordReg2(password : any) {
        let passwordreg = /^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d]{6,12}$/;
        let isValid = passwordreg.test(password);
        return isValid;
    },
    // 至少六个字符，纯数字验证
    passwordReg1(password : any) {
        let passwordreg = /^[0-9]{6,12}$/;
        let isValid = passwordreg.test(password);
        return isValid;
    },
    // 去掉中文及空格
    onVerifyCnAndSpace(val: string) {
        // 输入框只能输入中文、字母和下横线
        let v = val.replace(/([^\u4E00-\u9FA5A-Za-z\uFF0E_](?!\u3000))+$/g, '');
        // 输入框只能输入字母和下横线
        // let v = val.replace(/[^a-zA-Z_]/g, '');
        return v;
        // 输入框只能输入字母数字和下横线(不能用数字开头)
        // let vv = val.replace(/^[0-9]/g, '');
        // vv = vv.replace(/[^\w]/g, '');
        // return vv;
    },
    // 限制公式格式只能四则运算
    onVerifyFormula(val: string) {
        let v = val.replace(/([^\u4E00-\u9FA5\(\)\.\+\-\*\/\#a-zA-Z0-9\uFF0E_](?!\u3000))+$/g, '');
        // let v = val.replace(/[^\(\)\.\+\-\*\/\#a-zA-Z0-9_]/g, '');
        return v;
    },
    // 正整数，可以小数点，不能是负数
    verifyNumberIntegerAndFloat(val: string) {
        // 匹配空格
        let v = val.replace(/(^\s*)|(\s*$)/g, '');
        // 只能是数字和小数点，不能是其他输入
        v = v.replace(/[^\d.]/g, '');
        // 以0开始只能输入一个
        v = v.replace(/^0{2}$/g, '0');
        // 保证第一位只能是数字，不能是点
        v = v.replace(/^\./g, '');
        // 保证小数点后面不能出现0，只能[1-9]
        // v = v.replace(/[^\d]0$/g, '');
        // 小数只能出现1位
        // v = v.replace('.', '$#$').replace(/\./g, '').replace('$#$', '.');
        // 小数点后面保留2位
        // v = v.replace(/^(\-)*(\d+)\.(\d).*$/, '$1$2.$3');
        // 返回结果
        return v;
    },
    /**
     * 正整数验证
     * @param val 当前值字符串
     * @returns 返回处理后的字符串
     */
    verifiyNumberInteger(val: string) {
        // 匹配空格
        let v = val.replace(/(^\s*)|(\s*$)/g, '');
        // 保证第一位只能是数字，不能是点
	    v = v.replace(/^\./g, '');
        // 去掉 '.' , 防止贴贴的时候出现问题 如 0.1.12.12
        v = v.replace(/[\.]*/g, '');
        // 去掉以 0 开始后面的数, 防止贴贴的时候出现问题 如 00121323
        v = v.replace(/(^0[\d]*)$/g, '0');
        // 首位是0,只能出现一次
        v = v.replace(/^0\d$/g, '0');
        // 只匹配数字
        v = v.replace(/[^\d]/g, '');
        // 返回结果
        return v;
    },
    // 检测是否为空
    emptyReg(obj : any) {
        let regu = "^[ ]+$";
        let re = new RegExp(regu);
        if(typeof obj == "undefined" || obj == null || obj == "" || re.test(obj)){
            return false;
        }
        return true;
    },
    // 省级地址码校验
    checkProv(val : any) {
        let pattern = /^[1-9][0-9]/;
        let provs = {11:"北京",12:"天津",13:"河北",14:"山西",15:"内蒙古",21:"辽宁",22:"吉林",23:"黑龙江 ",31:"上海",32:"江苏",33:"浙江",34:"安徽",35:"福建",36:"江西",37:"山东",41:"河南",42:"湖北 ",43:"湖南",44:"广东",45:"广西",46:"海南",50:"重庆",51:"四川",52:"贵州",53:"云南",54:"西藏 ",61:"陕西",62:"甘肃",63:"青海",64:"宁夏",65:"新疆",71:"中国台湾",81:"中国香港",82:"中国澳门"};
        if(pattern.test(val)) {
            if(provs[val]) {
                return true;
            }
        }
        return false;
    },
    // 出生日期码校验
    checkDate(val : any) {
        let pattern = /^(18|19|20)\d{2}((0[1-9])|(1[0-2]))(([0-2][1-9])|10|20|30|31)$/;
        if(pattern.test(val)) {
            let year = val.substring(0, 4);
            let month = val.substring(4, 6);
            let date = val.substring(6, 8);
            let date2 = new Date(year+"-"+month+"-"+date);
            if(date2 && date2.getMonth() == (parseInt(month) - 1)) {
                return true;
            }
        }
        return false;
    },
    // 校验码校验
    checkCode(val : any) {
        let p = /^[1-9]\d{5}(18|19|20)\d{2}((0[1-9])|(1[0-2]))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]$/;
        let factor = [ 7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2 ];
        let parity = [ 1, 0, 'X', 9, 8, 7, 6, 5, 4, 3, 2 ];
        let code = val.substring(17);
        if(p.test(val)) {
            let sum = 0;
            for(let i=0;i<17;i++) {
                sum += val[i]*factor[i];
            }
            if(parity[sum % 11] == code.toUpperCase()) {
                return true;
            }
        }
        return false;
    },
    // 身份证校验
    checkID(val : any) {
        if(this.checkCode(val)) {
            let date = val.substring(6,14);
            if(this.checkDate(date)) {
                if(this.checkProv(val.substring(0,2))) {
                    return true;
                }
            }
        }
        return false;
    },
    // 根据身份证号自动计算年龄
    autoComputerID(e: any,name: string) {
        if(this.emptyReg(e) && this.checkID(e)){
            let date = new Date();
            let year = date.getFullYear();
            let month = date.getMonth()+1;
            let day = date.getDate();
            let initYear = e.substr(6,4);
            let initMonth = e.substr(10,2);
            let initDay = e.substr(12,2);
            initMonth = Number(initMonth)<10?initMonth.split('0')[1]:initMonth;
            initDay = Number(initDay)<10?initDay.split('0')[1]:initDay;
            let age = Number(year-initYear);
            if(Number(initMonth>month)){
                age = age-1;
            }else if(Number(initMonth==month)){
                if(initDay>day){
                    age = age-1;
                }else if(initDay==day&&this.emptyReg(name)){
                    console.log(`今天是${name}的生日，祝${name}生日快乐!`);
                    ElMessage.success(`今天是${name}的生日，祝${name}生日快乐!`);
                }
            }
            return age;
        }
    },
}
export default ruleReg;
